/*=============================================================================
Turing Designer
de_genw.cpp - Generic GUI windows

  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2005
  -- License restrictions apply (GPL v2)

  This file is part of Turing Designer.
  Turing Desigenr is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; however ONLY version 2 of the License.

  Turing Designer is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program kit; if not, write to:
  Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.

  -- For details see the COPYING file
=============================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#    include <wx/wx.h>
#endif
#include <wx/minifram.h>
#include <wx/statline.h>
#include "de_genw.hpp"
#include "xf_wxsu.hpp"

//------------------------------------------------------------------------------
GTButton::GTButton(wxWindow *parent, wxWindowID id, const char *title,
 const wxColour *cl, const char *tooltip, long style) :
    wxButton(parent, id, title, wxDefaultPosition, wxDefaultSize,
     style | wxBU_EXACTFIT)
{
    if(cl != NULL) { SetForegroundColour(*cl); }
    if(tooltip != NULL) { SetToolTip(tooltip); }
    return;
}

//-----------------------------------------------------------------------------
GWButton::GWButton(wxWindow *parent, wxWindowID id, const wxBitmap &bmp,
 const char *tooltip) :
    wxBitmapButton(parent, id, bmp)
{
    if(tooltip != NULL) { SetToolTip(tooltip); }
    return;
}

//-----------------------------------------------------------------------------
GWMenu::GWMenu(const wxString &title, long style) :
    wxMenu(title, style)
{
    return;
}

void GWMenu::Append_WIcon(int id, const wxString &text, const wxBitmap &icon,
 const char *help, wxItemKind kind) {
    wxMenuItem *item = new wxMenuItem(this, id, text, help, kind);
    item->SetBitmap(icon);
    Append(item);
    return;
}

//-----------------------------------------------------------------------------
BEGIN_EVENT_TABLE(GWMessage, wxDialog)
    EVT_BUTTON(wxID_YES, GWMessage::Yes)
    EVT_BUTTON(wxID_NO,  GWMessage::No)
END_EVENT_TABLE()

GWMessage::GWMessage(wxWindow *parent, const char *title, const char *prompt,
 long bt, long ts, wxWindowID id, const wxPoint &pos, const wxSize &size) :
    wxDialog(parent, id, title, pos, size, COMMON_FRAME_FLAGS)
{
    wxBoxSizer *vpacker = new wxBoxSizer(wxVERTICAL),
     *hpacker = new wxBoxSizer(wxHORIZONTAL);

    vpacker->Add(new wxStaticText(this, wxID_ANY, prompt, wxDefaultPosition,
     wxDefaultSize, ts), 0, wxALL | wxGROW, 10);
    vpacker->Add(-1, 1, 1);
    vpacker->Add(new wxStaticLine(this, wxID_ANY, wxDefaultPosition,
     wxDefaultSize, wxHORIZONTAL), 0, wxTOP | wxBOTTOM | wxGROW, 3);
    vpacker->Add(hpacker, 0, wxALIGN_RIGHT, 0);

    if(bt & wxOK) {
        hpacker->Add(new wxButton(this, wxID_OK, "OK"), 0,
         wxALIGN_CENTER | wxALL, 5);
    }
    if(bt & wxYES_NO) {
        hpacker->Add(new wxButton(this, wxID_YES, "Yes"), 0,
         wxALIGN_CENTER | wxALL, 5);
        hpacker->Add(new wxButton(this, wxID_NO, "No"), 0,
         wxALIGN_CENTER | wxALL, 5);
    }
    if(bt & wxCANCEL) {
        hpacker->Add(new wxButton(this, wxID_CANCEL, "Cancel"), 0,
         wxALIGN_CENTER | wxALL, 5);
    }

    SetSizer(vpacker);
    vpacker->SetSizeHints(this);
    SetSize(GetSize().GetWidth() + 50, -1);
    return;
}

void GWMessage::Yes(wxCommandEvent &ev) {
    EndModal(wxID_YES);
    return;
}

void GWMessage::No(wxCommandEvent &ev) {
    EndModal(wxID_NO);
    return;
}

//-----------------------------------------------------------------------------
GWText::GWText(wxWindow *parent, const char *title, const char *prompt,
 const char *std, bool multi, wxWindowID id, const wxPoint &pos,
 const wxSize &size) :
    wxDialog(parent, id, title, pos, size, COMMON_FRAME_FLAGS)
{
    wxBoxSizer *vpacker = new wxBoxSizer(wxVERTICAL),
     *hpacker = new wxBoxSizer(wxHORIZONTAL);
    field = new wxTextCtrl(this, wxID_ANY, std, wxDefaultPosition,
     wxDefaultSize, (multi * wxTE_MULTILINE) | wxTE_LEFT);

    vpacker->Add(new wxStaticText(this, wxID_ANY, prompt, wxDefaultPosition,
     wxDefaultSize, wxALIGN_LEFT), 0, wxALL | wxGROW, 5);
    vpacker->Add(field, 0, wxALL | wxGROW, 5);
    vpacker->Add(-1, 1, 1);
    vpacker->Add(new wxStaticLine(this, wxID_ANY, wxDefaultPosition,
     wxDefaultSize, wxHORIZONTAL), 0, wxTOP | wxBOTTOM | wxGROW, 3);
    vpacker->Add(hpacker, 0, wxALIGN_RIGHT, 0);

    hpacker->Add(new wxButton(this, wxID_OK, "OK"),
     0, wxALIGN_CENTER | wxALL, 5);
    hpacker->Add(new wxButton(this, wxID_CANCEL, "Cancel"),
     0, wxALIGN_CENTER | wxALL, 5);

    SetSizer(vpacker);
    vpacker->SetSizeHints(this);

    SetSize(GetSize().GetWidth() + 50, -1);
    return;
}

const char *GWText::get_value(void) {
    return field->GetValue();
}

//==[ End of file ]============================================================
