/*=============================================================================
Turing Designer
de_pad.hpp - Designer Drawpad

  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2005
  -- License restrictions apply (GPL v2)

  This file is part of Turing Designer.
  Turing Desigenr is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; however ONLY version 2 of the License.

  Turing Designer is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program kit; if not, write to:
  Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.

  -- For details see the COPYING file
=============================================================================*/
#ifndef DE_PAD_HPP
#define DE_PAD_HPP 1

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#    include <wx/wx.h>
#endif

enum {
    DWEXT_MARK = 1,
    DWEXT_SNODE,
    DWEXT_INODE,
    DWEXT_LNODE,
    DWEXT_LINK,
    DWEXT_LINK1,
    DWEXT_LINKD,
    DWEXT_ENTRYPT,
};

union bbox;
class TGraph;
class XVNode;
class XSNode;
class XINode;
class XLNode;

class Drawpad : public wxPanel {
  public: // functions
    Drawpad(wxWindow *, TGraph * = NULL);
    void Delete(wxCommandEvent &);
    void ext_action(unsigned int);
    
  private: // functions
    void Mouse_Enter(wxMouseEvent &);
    void Mouse_LeftDown(wxMouseEvent &);
    void Mouse_LeftUp(wxMouseEvent &);
    void Mouse_Motion(wxMouseEvent &);
    void Window_Erase(wxEraseEvent &);
    void Window_Paint(wxPaintEvent &);
    void Window_Resize(wxSizeEvent &);
    void Window_Scroll(wxScrollWinEvent &);
    void delete_targets(VNode *);
    void do_del_link(XVNode *, wxDC *);
    void do_link(XVNode *, int, wxDC *);
    void draw_arrow(XVNode *, XVNode *, wxDC *, int * = NULL,
      int * = NULL) const;
    void draw_arrows(XSNode *, int, int, int, int, wxDC *) const;
    template<class Object> void draw_arrows(Object *,
      int, int, int, int, wxDC *) const;
    void draw_node(XVNode *, int, int, int, wxDC *) const;
    union bbox *find_bounds(XVNode *, XVNode *, union bbox *) const;
    XVNode *find_node(int, int) const;
    int find_node_i(int, int) const;
    void select_node(unsigned int, unsigned int = 0);
    void switch_crosshair(void);

  private: // variables
    int vpx, vpy, vpw, vph, thx, thy;

    bool mouse_motion_oc, mouse_prev_sel;
    wxPoint mouse_prev_pos;
    int mouse_inode;

    unsigned int mode;
    TGraph *graph;
    XVNode *templink;

    DECLARE_EVENT_TABLE();
};

#endif // DE_PAD_HPP

//==[ End of file ]============================================================
