/*=============================================================================
Turing Designer
gr_data.cpp - S/I/L-Node and ATarget functions

  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2005
  -- License restrictions apply (GPL v2)

  This file is part of Turing Designer.
  Turing Desigenr is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; however ONLY version 2 of the License.

  Turing Designer is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program kit; if not, write to:
  Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.

  -- For details see the COPYING file
=============================================================================*/
#include <libHX.h>
#include "gr_data.hpp"

//-----------------------------------------------------------------------------
SNode::SNode(void) : VNode() {
    jmp = HXbtree_init(HXBT_MAP | HXBT_ICMP | HXBT_NCID);
    return;
}

SNode::~SNode(void) {
    struct HXbtree_node *node;
    void *travp = HXbtrav_init(jmp, NULL);
    while((node = HXbtraverse(travp)) != NULL) {
        delete reinterpret_cast<ATarget *>(node->data);
    }
    HXbtree_free(jmp);
    return;
}

void SNode::del(void *key) {
    ATarget *old = reinterpret_cast<ATarget *>(HXbtree_del(jmp, key));
    delete old;
    return;
}

void SNode::del(char key) {
    del(reinterpret_cast<void *>((int)(key)));
    return;
}

ATarget *SNode::get(char key) const {
    return static_cast<ATarget *>(HXbtree_get(jmp, (void *)(int)key));
}

void SNode::put(char i, char o, char mov, int next) {
    HXbtree_add(jmp, reinterpret_cast<void *>((int)i),
     new ATarget(o, mov, next));
    return;
}

//-----------------------------------------------------------------------------
INode::INode(int next, TGraph *data) : VNode() {
    this->next  = next;
    this->graph = data;
    return;
}

INode::~INode(void) {
    delete ext;
    delete graph;
    return;
}

//-----------------------------------------------------------------------------
LNode::LNode(int next) : VNode() {
    this->next = next;
    return;
}

//-----------------------------------------------------------------------------
ATarget::ATarget(char out, char mov, int next) {
    this->out  = out;
    this->mov  = mov;
    this->next = next;
    return;
}

//==[ End of file ]============================================================
