/*=============================================================================
Turing Designer
gr_data.hpp - Node and graph definitions

  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2005
  -- License restrictions apply (GPL v2)

  This file is part of Turing Designer.
  Turing Desigenr is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; however ONLY version 2 of the License.

  Turing Designer is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program kit; if not, write to:
  Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.

  -- For details see the COPYING file
=============================================================================*/
#ifndef GR_DATA_HPP
#define GR_DATA_HPP 1

#include <sys/types.h>
#include <stdio.h>
#include <libHX.h>

enum {
    TYPE_INVALID = 0,
    TYPE_SNODE,
    TYPE_INODE,
    TYPE_LNODE,
    MOV_NONE = 0,
    MOV_LEFT,
    MOV_RIGHT,
};

class ATarget;
class TGraph;

class VNode {
    /* Almost empty class serving as an inheritance point and also to allow
    lesser-type function argument passing (i.e. func(VNode *), but can be
    passed an SNode). */
  public: // fucntions
    virtual const unsigned int type(void) const = NULL;
};

class SNode : public virtual VNode {
  public: // functions
    SNode(void);
    virtual ~SNode(void);
    void del(void *);
    void del(char);
    ATarget *get(char) const;
    void put(char = 0, char = 0, char = 0, int = -1);
    inline const unsigned int type(void) const { return TYPE_SNODE; };

  public: // variables
    struct HXbtree *jmp;
};

class INode : public virtual VNode {
  public: // functions
    INode(int = -1, TGraph * = NULL);
    virtual ~INode(void);
    inline const unsigned int type(void) const { return TYPE_INODE; };

  public: // variables
    const char *ext;
    TGraph *graph;
    int next;
};

class LNode : public virtual VNode {
  public: // functions
    LNode(int = -1);
    inline const unsigned int type(void) const { return TYPE_LNODE; };

  public: // variables
    int next;
};

class ATarget {
  public: // functions
    ATarget(char = 0, char = 0, int = -1);

  public: // variables
    char out, mov;
    int next;
};

class TGraph {
  public: // functions
    static void *operator new(size_t);
    TGraph(unsigned int = Default_size);
    ~TGraph(void);
    inline void operator ()(unsigned int n) { alloc_nodes(n); return; }
    VNode *&operator [](unsigned int);

    int add(SNode *);
    int add(VNode *);
    void del(unsigned int);
    inline unsigned int last_idx(void) const { return alloc_count - 1; }
    VNode *walk(int *) const;
    VNode *walk_rev(int *) const;

  public: // variables
    unsigned int entrypoint;
    bool modified;

    static const size_t Default_size = 16;

  protected: // functions
    void alloc_nodes(unsigned int);
    int enqueue(VNode *);

  protected: // variables
    unsigned int node_count, alloc_count;
    VNode **node;
};

#endif // GR_DATA_HPP

//==[ End of file ]============================================================
