/*=============================================================================
Turing Designer
gr_tgrap.cpp - TGraph functions

  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2005
  -- License restrictions apply (GPL v2)

  This file is part of Turing Designer.
  Turing Desigenr is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; however ONLY version 2 of the License.

  Turing Designer is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program kit; if not, write to:
  Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.

  -- For details see the COPYING file
=============================================================================*/
#include <sys/types.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gr_data.hpp"

//-----------------------------------------------------------------------------
void *TGraph::operator new(size_t s) {
    return calloc(1, s);
}

TGraph::TGraph(unsigned int n) {
    alloc_nodes(n);
    return;
}

TGraph::~TGraph(void) {
    for(size_t n = 0; n < alloc_count; ++n) {
        delete node[n];
    }
    free(node);
    return;
}

VNode *&TGraph::operator [](unsigned int i) {
    if(i == -1) {
        fprintf(stderr, "%s: Someone set us up the bomb!\n",
         __PRETTY_FUNCTION__);
    }
    modified = true;
    return node[i];
}

int TGraph::add(SNode *node) {
    return enqueue(node);
}

int TGraph::add(VNode *node) {
    return enqueue(node);
}

void TGraph::del(unsigned int n) {
    delete node[n];
    node[n] = NULL;
    modified = true;
    return;
}

VNode *TGraph::walk(int *pos) const {
    while(*pos < alloc_count) {
        if(node[*pos] != NULL) {
            return node[(*pos)++];
        }
        ++*pos;
    }
    return NULL;
}

VNode *TGraph::walk_rev(int *pos) const {
    while(*pos >= 0) {
        if(node[*pos] != NULL) {
            return node[(*pos)--];
        }
        --*pos;
    }
    return NULL;
}

//-----------------------------------------------------------------------------
void TGraph::alloc_nodes(unsigned int hint) {
    /* If we have more than HINT nodes allocated, do nothing. Otherwise, double
    the number of buckets, and if that's still not sufficient, increase to the
    HINT value. Finally, zero out the newly allocated buckets. */
    if(hint > 0 && alloc_count > hint) {
        return;
    }

    int pre = alloc_count;
    if(alloc_count == 0) {
        alloc_count = 1; // to make *= 2 valid
    }
    alloc_count *= 2;
    if(hint > alloc_count) {
        alloc_count = hint;
    }
    node = (VNode **)realloc(node, sizeof(VNode *) * alloc_count);
    memset(&node[pre], 0, sizeof(VNode *) * (alloc_count - pre));
    return;
}

int TGraph::enqueue(VNode *n) {
    int c = 0;
    while(1) {
        for(size_t i = 0; i < alloc_count; ++i) {
            if(node[i] == NULL) {
                node[i] = n;
                ++node_count;
                modified = true;
                return i;
            }
        }
        if(c++) { return -1; }
        alloc_nodes(0); // 0 = no check
    }
    return -1;
}

//==[ End of file ]============================================================
