/*=============================================================================
Turing Designer
si_sim.hpp - GUI Simulator Engine

  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2005
  -- License restrictions apply (GPL v2)

  This file is part of Turing Designer.
  Turing Desigenr is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; however ONLY version 2 of the License.

  Turing Designer is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program kit; if not, write to:
  Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.

  -- For details see the COPYING file
=============================================================================*/
#ifndef SI_SIM_HPP
#define SI_SIM_HPP 1

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#    include <wx/wx.h>
#endif

class TGraph;
class TuMach;
class SimTape;

class Simulator : public wxPanel {
  public: // functions
    Simulator(wxWindow *, TGraph *);
    ~Simulator(void);
    void Run(wxCommandEvent &);
    void Step_One(wxCommandEvent &);
    void Step_Until(wxCommandEvent &);
    void Step_Many(wxCommandEvent &);
    void Stop(wxCommandEvent &);
    void Reset(wxCommandEvent &);
    void show_next_step(void);

  public: // variables
    TGraph *graph;

  private: // functions
    void Timer(wxTimerEvent &);
    void bp_del(unsigned int);
    bool bp_get(unsigned int);
    bool bp_put(unsigned int);
    void bp_extend(size_t);
    int do_tick(void);
    int get_delay(void);
    int get_steps(void);
    wxStaticBoxSizer *gen_machview(void);

  private: // variables
    TuMach *mach;
    SimTape *tape;
    int steps_left;

    wxTextCtrl *tx_state, *tx_step, *tx_delay;
    wxCheckBox *cb_step;
    wxTimer *wtimer;

    unsigned char *bp_map;
    size_t bp_size;

    DECLARE_EVENT_TABLE();
};

#endif // SI_SIM_HPP

//==[ End of file ]============================================================
