/*=============================================================================
Turing Designer
tu_tape.cpp - UI-neutral Tape implementation

  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2005
  -- License restrictions apply (GPL v2)

  This file is part of Turing Designer.
  Turing Desigenr is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; however ONLY version 2 of the License.

  Turing Designer is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program kit; if not, write to:
  Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.

  -- For details see the COPYING file
=============================================================================*/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libHX.h>
#include "tu_tape.hpp"

//-----------------------------------------------------------------------------
TuTape::TuTape(const char *input, int pos) {
    if(input != NULL) {
        data = HX_strdup(input);
        dlen = strlen(data);
        head_pos = pos;
    } else {
        data = NULL;
        dlen = 0;
        head_pos = 0;
    }
    return;
}

TuTape::~TuTape(void) {
    free(data);
    return;
}

int TuTape::operator --(void) {
    if(head_pos == 0) { return -1; }
    return --head_pos;
}

int TuTape::operator ++(void) {
    extend(++head_pos);
    return head_pos;
}

void TuTape::update(void) {
    return;
}

//-----------------------------------------------------------------------------
void TuTape::extend(size_t pos) {
    if(dlen < pos + 1) {
        data = reinterpret_cast<char *>(realloc(data, pos + 2));
        memset(&data[dlen], '_', pos - dlen);
        data[pos + 1] = '\0';
        dlen = pos + 1;
    }
    return;
}

//==[ End of file ]============================================================
