/*=============================================================================
Turing Designer
xf_main.hpp - GUI MainWindow

  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2005
  -- License restrictions apply (GPL v2)

  This file is part of Turing Designer.
  Turing Desigenr is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; however ONLY version 2 of the License.

  Turing Designer is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program kit; if not, write to:
  Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.

  -- For details see the COPYING file
=============================================================================*/
#ifndef XF_MAIN_HPP
#define XF_MAIN_HPP 1

#include <stdio.h>
#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#    include <wx/wx.h>
#endif

class VNode;
class TGraph;
class Drawpad;
class Simulator;

class MainWindow : public wxFrame {
  public: // functions
    MainWindow(const char * = NULL, const wxPoint & = wxDefaultPosition,
     const wxSize & = wxDefaultSize, TGraph * = NULL);
    ~MainWindow(void);

  private: // functions
    void File_New(wxCommandEvent &);
    void File_Open(wxCommandEvent &);
    void File_Save(wxCommandEvent &);
    void File_SaveAs(wxCommandEvent &);
    void File_Exit(wxCommandEvent &);
    void Edit_Delete(wxCommandEvent &);
    void Edit_Entrypoint(wxCommandEvent &);
    void Tool_Mark(wxCommandEvent &);
    void Tool_SNode(wxCommandEvent &);
    void Tool_INode(wxCommandEvent &);
    void Tool_LNode(wxCommandEvent &);
    void Tool_Link(wxCommandEvent &);
    void Tool_DelLink(wxCommandEvent &);
    void Run_Run(wxCommandEvent &);
    void Run_Step(wxCommandEvent &);
    void Run_Stop(wxCommandEvent &);
    void Run_Reset(wxCommandEvent &);
    void About(wxCommandEvent &);
    void Roll_Down(wxCommandEvent &);
    void Roll_Up(wxCommandEvent &);
    void Roll_Equal(wxCommandEvent &);
    int check_modified(const char * = NULL);
    wxBoxSizer *create_drawbar(void);
    int save_graph(bool);
    int save_graph_as(bool);

  private: // variables
    char *filename;
    wxBoxSizer *packer, *drw;
    TGraph *graph;
    Drawpad *pad;
    Simulator *sim;

    DECLARE_EVENT_TABLE();
};

extern MainWindow *new_mw(TGraph * = NULL);

#endif // XF_MAIN_HPP

//==[ End of file ]============================================================
